#include <gltk.hh>
#include <iostream>

namespace Trackball {
        extern "C" {
#include "trackball.h"
        }
}


struct MData {
	float Pos[3];
	float Quat[4];
	float Scale;

	float QuatDiff[4];
	float BeginX;
	float BeginY;
};


class MyArea :
	public gltk::glArea
{
	MData mdata;

	void transform()
	{
		glTranslated(mdata.Pos[0], mdata.Pos[1], mdata.Pos[2]);
		glScaled(mdata.Scale, mdata.Scale, mdata.Scale);
		float r[4][4];
		Trackball::build_rotmatrix(r, mdata.Quat);
		glMultMatrixf(& r[0][0]);
	}

	void gl_setup()
	{
		float pos[4] = {5.0, 10.0, 5.0, 0.0};
		glClearColor(1.0, 1.0, 1.0, 0.0);
		glLightfv(GL_LIGHT0, GL_POSITION, pos);
		glEnable(GL_CULL_FACE);
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_DEPTH_TEST);
		glEnable(GL_NORMALIZE);
		glEnable(GL_COLOR_MATERIAL);
		float am[4] = {0.1, 0.2, 0.15, 1.0};
		float di[4] = {0.5, 0.4, 0.5, 1.0};
		float sp[4] = {0.4, 0.5, 0.5, 1.0};
		float sh = 20.0;
		glMaterialfv(GL_FRONT, GL_AMBIENT, am);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, di);
		glMaterialfv(GL_FRONT, GL_SPECULAR, sp);
		glMaterialf(GL_FRONT, GL_SHININESS, sh);
		glEndList();
	}

	void gl_new_size(int w, int h)
	{
		glViewport(0, 0, w, h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		if (w > h) { // adjust the aspect ratio
			double a = w / (double) h;
			glFrustum(- a, a, - 1.0, 1.0, 4.0, 60.0);
		}
		else {
			double a = h / (double) w;
			glFrustum(- 1.0, 1.0, - a, a, 4.0, 60.0);
		}
		glMatrixMode(GL_MODELVIEW);
	}

	void gl_draw()
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		transform();
		// draw a box
		glLineWidth(1);
		glBegin(GL_LINES);
		glColor3d(0.0, 0.0, 0.5);
		glVertex3d(1.0, 1.0, 1.0);
		glVertex3d(1.0, 1.0, - 1.0);
		glVertex3d(1.0, 1.0, 1.0);
		glVertex3d(1.0, - 1.0, 1.0);
		glVertex3d(1.0, 1.0, 1.0);
		glVertex3d(- 1.0, 1.0, 1.0);
		glVertex3d(1.0, 1.0, - 1.0);
		glVertex3d(1.0, - 1.0, - 1.0);
		glVertex3d(1.0, 1.0, - 1.0);
		glVertex3d(- 1.0, 1.0, - 1.0);
		glVertex3d(1.0, - 1.0, 1.0);
		glVertex3d(1.0, - 1.0, - 1.0);
		glVertex3d(1.0, - 1.0, 1.0);
		glVertex3d(- 1.0, - 1.0, 1.0);
		glVertex3d(- 1.0, 1.0, 1.0);
		glVertex3d(- 1.0, 1.0, - 1.0);
		glVertex3d(- 1.0, 1.0, 1.0);
		glVertex3d(- 1.0, - 1.0, 1.0);
		glVertex3d(1.0, - 1.0, - 1.0);
		glVertex3d(- 1.0, - 1.0, - 1.0);
		glVertex3d(- 1.0, 1.0, - 1.0);
		glVertex3d(- 1.0, - 1.0, - 1.0);
		glVertex3d(- 1.0, - 1.0, 1.0);
		glVertex3d(- 1.0, - 1.0, - 1.0);
		glEnd();
	}

	int down_button;

	void on_mouse_func(int button, int state, int x, int y)
	{
		if (state == GLUT_DOWN) {
			down_button = button;
			mdata.BeginX = x;
			mdata.BeginY = y;
		}
	}

	void on_motion_func(int x, int y)
	{
		double w = get_width();
		double h = get_height();
		bool redraw = false;
		// Rotation.
		if (down_button == GLUT_LEFT_BUTTON) {
			Trackball::trackball(mdata.QuatDiff, (2 * mdata.BeginX - w) / w, (h - 2 * mdata.BeginY) / h, (2 * x - w) / w, (h - 2 * y) / h);
			Trackball::add_quats(mdata.QuatDiff, mdata.Quat, mdata.Quat);
			redraw = true;
		}
		// Scaling.
		else if (down_button == GLUT_RIGHT_BUTTON) {
			mdata.Scale = mdata.Scale * (1.0 + (y - mdata.BeginY) / h);
			if (mdata.Scale > 2.0) mdata.Scale = 2.0;
			else if (mdata.Scale < 0.5) mdata.Scale = 0.5;
			redraw = true;
		}
		mdata.BeginX = x;
		mdata.BeginY = y;
		if (redraw) glutPostRedisplay();
	}
public:
	MyArea()
	{
		mdata.Scale = 1;
		mdata.BeginX = 0;
		mdata.BeginY = 0;

		// reset
		mdata.Pos[0] = 0;
		mdata.Pos[1] = 0;
		mdata.Pos[2] = - 10;
		mdata.Quat[0] = 0;
		mdata.Quat[1] = 0;
		mdata.Quat[2] = 0;
		mdata.Quat[3] = 1;
		mdata.Scale = 1;
		mdata.QuatDiff[0] = 0;
		mdata.QuatDiff[1] = 0;
		mdata.QuatDiff[2] = 0;
		mdata.QuatDiff[3] = 1;
		add_events(glutWindow::MOUSE_MASK | glutWindow::MOTION_MASK);
	}
};

int main(int argc, char *argv[])
{
	gltk::Main kit(argc, argv);
	// glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	gltk::Window window;
	window.set_title("This is a test");
	MyArea area;
	area.set_size(200, 200);
	area.show();
	window.add(& area);
	// window.show();
	gltk::Main::run(window);
	return 0;
}
