/* bin.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_BIN_HH
#define GLTK_BIN_HH
#include "container.hh"
namespace gltk {
	/** Bin is a Container with a single child.
	    Bin maintains a reducible border around a single child in a
	    designated area of the allocated space. The reducible border is
	    specified by the #set_reducible function. The area outside the
	    reducible boarder is set by #set_reserved and remains fixed.
	    The size are allocated in the order of precedence:

	    reserved > minimum of child > natural of child > reducible

	    The minimum size of the Bin is the reserved + minimum of child.
	    The natural size is reserved + natural of child + reducible. */
	class Bin :
		public virtual Container
	{
		// internal variables
		int child_x;
		int child_y;
	protected:
		// overrides
		void on_configure_event() /*override*/;

		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;
		void get_preferred_width_for_height(int height, int & min, int & nat) const;
		void get_preferred_height_for_width(int width, int & min, int & nat) const;

		void on_add(Widget *);
		void on_remove(Widget *);

		Widget * child;
		int reducible_x;  ///< space for the reducible border
		int reducible_y;  ///< space for the reducible border
		int reserved_x; ///< reserved space in x
		int reserved_y; ///< reserved space in y
	public:
		Bin();

		void set_reducible(int x, int y); ///< set reducible spaces
		void set_reserved(int x, int y); ///< set reserved spaces
		void place_bin(int x, int y); ///< move the Bin within the Widget
		void set_border_width(unsigned w);
	};
}
#endif
