/* button.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_BUTTON_HH
#define GLTK_BUTTON_HH
#include "pushbox.hh"
#include "labelbin.hh"
#include "sigc.hh"
#include <string>
namespace gltk {
	/** Button is a PushBox that holds a child widget.
	    The child widget is moved by (1,1) when the PushBox is depressed.
	    \image html Button.png */
	class Button :
		virtual public PushBox,
		virtual public LabelBin
	{
		// overrides
		void on_draw();

		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;
		void get_preferred_width_for_height(int height, int & min, int & nat) const;
		void get_preferred_height_for_width(int width, int & min, int & nat) const;

		bool focus_make();
		bool focus_next(); // need to override that from Bin
		void focus_end();

		void on_button_event(int button, int state, int x, int y);
		bool on_key_event(unsigned char key);
	public:
		/// initialize Button with a text label
		Button(std::string const & name);
		/// initialize Button with a child widget
		Button(Widget * widget);
	};
}
#endif
