/* checkbox.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "checkbox.hh"
#include "glut_inc.hh"
using namespace gltk;

const int CheckBox::checkbox_size = 7;
const int CheckBox::checkbox_area = 20;

void CheckBox::on_draw()
{
	draw_focus_box();
	draw_checkbox(0, 0, get_width(), get_height());
}

void CheckBox::get_preferred_width(int & min, int & nat) const
{
	min = checkbox_size;
	nat = checkbox_area;
}

void CheckBox::get_preferred_height(int & min, int & nat) const
{
	min = checkbox_size;
	nat = checkbox_area;
}

void CheckBox::on_button_event(int button, int state, int x, int y)
{
	if (button != GLUT_LEFT_BUTTON) return;
	if (state == GLUT_DOWN) {
		active = ! active;
		toggled.emit();
	}
	queue_draw();
}

bool CheckBox::on_key_event(unsigned char key)
{
	if (key == 0xd || key == 0x20) {
		active = ! active;
		toggled.emit();
		queue_draw();
		return true;
	}
	return false;
}

void CheckBox::draw_checkbox(int x, int y, int w, int h)
{
	int checkbox_size = 7;
	if (w > checkbox_size && h > checkbox_size) {
		float dx = (w - checkbox_size) / 2 - 0.5;
		float dy = (h - checkbox_size) / 2 - 0.5;
		glColor3ub(0, 0, 0);
		glBegin(GL_LINE_LOOP);
		glVertex2f(dx, dy);
		glVertex2f(dx + checkbox_size, dy);
		glVertex2f(dx + checkbox_size, dy + checkbox_size);
		glVertex2f(dx, dy + checkbox_size);
		glEnd();
		if (active) {
			glColor3ub(0, 0, 0);
			glBegin(GL_LINES);
			glVertex2f(dx, dy);
			glVertex2f(dx + checkbox_size, dy + checkbox_size);
			glVertex2f(dx + checkbox_size, dy);
			glVertex2f(dx, dy + checkbox_size);
			glEnd();
		}
	}
}

CheckBox::CheckBox() :
	active(false)
{
}

sigc::signal<void> & CheckBox::signal_toggled()
{
	return toggled;
}

bool CheckBox::get_active() const
{
	return active;
}

void CheckBox::set_active(bool val)
{
	active = val;
}
