/* glarea.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "glarea.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include "window.hh"
#include <iostream>
#include <map>
#include <cassert>

using namespace gltk;

void glArea::on_display_func()
{
	// glutSetWindow(get_win_id());
	assert(glutGetWindow() == get_win_id());
	gl_draw();
	glutSwapBuffers();
}

void glArea::on_reshape_func(int width, int height)
{
	gl_new_size(width, height);
}

void glArea::on_draw()
{
	if (int wid = get_win_id()) {
		int oid = glutGetWindow();
		glutSetWindow(wid);
		glutPostRedisplay();
		glutSetWindow(oid);
	}
}

void glArea::on_unrealize()
{
	if (get_win_id()) destroy();
}

void glArea::on_configure_event()
{
	if (! get_shown()) return;
	Allocation a = get_allocation();
	if (int wid = get_win_id()) {
		int oid = glutGetWindow();
		glutSetWindow(wid);
		glutReshapeWindow(a.width, a.height);
		glutPositionWindow(a.x, a.y);
		glutSetWindow(oid);
	}
	else {
		Window * w = get_window();
		assert(w);
		int pwin = w->get_id();
		assert(pwin);
		create(pwin, a.x, a.y, a.width, a.height);
		gl_setup();
	}
}

void glArea::on_show()
{
	if (int pwin = get_realized()) {
		if (int wid = get_win_id()) {
			glutSetWindow(wid);
			glutShowWindow();
		}
		else {
			Allocation a = get_allocation();
			create(pwin, a.x, a.y, a.width, a.height);
			gl_setup();
		}
	}
}

void glArea::on_hide()
{
	if (int wid = get_win_id()) {
		glutSetWindow(wid);
		glutHideWindow();
	}
}

void glArea::get_preferred_height(int & min, int & nat) const
{
	min = 0;
	nat = area_height;
}

void glArea::get_preferred_width(int & min, int & nat) const
{
	min = 0;
	nat = area_width;
}

// The Contructor

glArea::glArea() :
	area_width(200),
	area_height(200)
{
	set_events(DISPLAY_MASK | RESHAPE_MASK | MOUSE_MASK | MOTION_MASK);
}

glArea::~glArea()
{
	if (get_win_id()) destroy();
}

void glArea::set_size(int width, int height)
{
	area_width = width;
	area_height = height;
}
