/* internal.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "glut_inc.hh"
#include "internal.hh"
#include <map>

namespace {
	struct glutFontInfo
	{
		void * font;
		int height;
		int ascent;
		int descent;
	} gf_info [] = {
		{GLUT_BITMAP_8_BY_13, 13, 11, 2},
		{GLUT_BITMAP_9_BY_15, 15, 13, 2},
		{GLUT_BITMAP_TIMES_ROMAN_10, 10, 7, 3},
		{GLUT_BITMAP_TIMES_ROMAN_24, 24, 18, 6},
		{GLUT_BITMAP_HELVETICA_10, 10, 8, 2},
		{GLUT_BITMAP_HELVETICA_12, 12, 9, 3},
		{GLUT_BITMAP_HELVETICA_18, 18, 15, 3},
		{0, 0, 0, 0}
	};
	glutFontInfo const & get_info(void * font)
	{
		size_t i = 0;
		while (gf_info[i].font && font != gf_info[i].font) i ++;
		return gf_info[i];
	}
}

using namespace gltk;
int gltk::glut_get_font_height(void * font)
{
	return get_info(font).height;
}

int gltk::glut_get_font_ascent(void * font)
{
	return get_info(font).ascent;
}

int gltk::glut_get_font_descent(void * font)
{
	return get_info(font).descent;
}

void gltk::glut_draw_string(void * font, std::string const & text)
{
	for (size_t i = 0; i < text.size(); i ++) glutBitmapCharacter(font, text[i]);
}

void gltk::glut_draw_string(void * font, std::string const & text, int limit)
{
	int w = 0;
	for (size_t i = 0; i < text.size(); i ++) {
		w += glutBitmapWidth(font, text[i]);
		if (w > limit) return;
		glutBitmapCharacter(font, text[i]);
	}
}

int gltk::glut_get_string_width(void * font, std::string const & text)
{
	int w = 0;
	for (size_t i = 0; i < text.size(); i ++) w += glutBitmapWidth(font, text[i]);
	return w;
}

void gltk::glut_draw_raised_box(int x, int y, int w, int h)
{
	w = w + x;
	h = h + y;

	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINE_STRIP);
	glVertex2f(x + 0.5, h - 0.5);
	glVertex2f(x + 0.5, y + 0.5);
	glVertex2f(w - 0.5, y + 0.5);
	glEnd();

	glColor3f(0.8, 0.8, 0.8);
	glBegin(GL_LINE_STRIP);
	glVertex2f(x + 1.5, h - 1.5);
	glVertex2f(x + 1.5, y + 1.5);
	glVertex2f(w - 1.5, y + 1.5);
	glEnd();

	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_LINE_STRIP);
	glVertex2f(w - 0.5, y + 0.5);
	glVertex2f(w - 0.5, h - 0.5);
	glVertex2f(x + 0.5, h - 0.5);
	glEnd();

	glColor3f(0.8, 0.8, 0.8);
	glBegin(GL_LINE_STRIP);
	glVertex2f(w - 1.5, y + 1.5);
	glVertex2f(w - 1.5, h - 1.5);
	glVertex2f(x + 1.5, h - 1.5);
	glEnd();
}

int gltk::glut_raised_box_width()
{
	return 2;
}

void gltk::glut_draw_sunken_box(int x, int y, int w, int h)
{
	w = w + x;
	h = h + y;

	glColor3f(0.8, 0.8, 0.8);
	glBegin(GL_LINE_STRIP);
	glVertex2f(x + 0.5, h - 0.5);
	glVertex2f(x + 0.5, y + 0.5);
	glVertex2f(w - 0.5, y + 0.5);
	glEnd();

	glColor3f(0.4, 0.4, 0.4);
	glBegin(GL_LINE_STRIP);
	glVertex2f(x + 1.5, h - 1.5);
	glVertex2f(x + 1.5, y + 1.5);
	glVertex2f(w - 1.5, y + 1.5);
	glEnd();

	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINE_STRIP);
	glVertex2f(w - 0.5, y + 0.5);
	glVertex2f(w - 0.5, h - 0.5);
	glVertex2f(x + 0.5, h - 0.5);
	glEnd();

	glColor3f(0.8, 0.8, 0.8);
	glBegin(GL_LINE_STRIP);
	glVertex2f(w - 1.5, y + 1.5);
	glVertex2f(w - 1.5, h - 1.5);
	glVertex2f(x + 1.5, h - 1.5);
	glEnd();
}

int gltk::glut_sunken_box_width()
{
	return 2;
}

