/* pushbox.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "pushbox.hh"
#include "label.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <iostream>
using namespace gltk;

void PushBox::on_draw()
{
	draw_box();
}

void PushBox::get_preferred_width(int & min, int & nat) const
{
	min = box_width * 2;
	nat = min + 10;
}

void PushBox::get_preferred_height(int & min, int & nat) const
{
	min = box_width * 2;
	nat = min + 10;
}

void PushBox::on_button_event(int button, int state, int x, int y)
{
	if (button != GLUT_LEFT_BUTTON) return;
	if (state == GLUT_DOWN) {
		depressed = true;
		set_grabbing();
		pushed.emit();
	}
	else if (depressed) {
		set_grabbing(false);
		depressed = false;
		if (x >= 0 && y >= 0 && x < get_width() && y < get_height()) clicked.emit();
	}
	queue_draw();
}

bool PushBox::on_key_event(unsigned char key)
{
	if (key == 0xd) {
		pushed.emit();
		clicked.emit();
		return true;
	}
	return false;
}

void PushBox::draw_box()
{
	int w = get_width();
	int h = get_height();
	int bw = glut_raised_box_width();
	glColor3f(0.9, 0.9, 0.9);
	glBegin(GL_QUADS);
	glVertex2i(bw, bw);
	glVertex2i(w - bw, bw);
	glVertex2i(w - bw, h - bw);
	glVertex2i(bw, h - bw);
	glEnd();
	// glPushMatrix();
	if (depressed) {
		glColor3f(0.6, 0.6, 0.6);
		glBegin(GL_LINE_LOOP);
		glVertex2f(0.5, 0.5);
		glVertex2f(w - 0.5, 0.5);
		glVertex2f(w - 0.5 , h - 0.5);
		glVertex2f(0.5, h - 0.5);
		glEnd();
	}
	else glut_draw_raised_box(0, 0, w, h);
	// glPopMatrix();
	if (focused) {
		glEnable(GL_LINE_STIPPLE);
		glLineStipple(1, 0x5555);
		glColor3f(0., 0., 0.);
		glBegin(GL_LINE_LOOP);
		double d = bw + 0.5;
		glVertex2f(d, d);
		glVertex2f(w - d, d);
		glVertex2f(w - d, h - d);
		glVertex2f(d, h - d);
		glEnd();
		glDisable(GL_LINE_STIPPLE);
	}
}

PushBox::PushBox() :
	box_width(4),
	depressed(false)
{}
