/* scale.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_SCALE_HH
#define GLTK_SCALE_HH
#include "bin.hh"
#include "textdraw.hh"
#include "positiontype.hh"
#include "slider.hh"
#include "label.hh"
namespace gltk {
	/** a slider to select and a label to display a numeric value.
	    It takes an Adjustment for manipulation.
	    \image html Scale.png */
	class Scale :
		TextDraw,
		public virtual Bin
	{
		Slider sl;
		PositionType pos;
		int digits;
		int wtxt; // width of label
		bool draw_value;
		void update_label();
		void update_wtxt();

		// overrides
		void on_configure_event();
		void on_draw();
		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_width_for_height(int height, int & min, int & nat) const;
	public:
		Scale(
			Adjustment * adjust = 0,
			Orientation orient = ORIENTATION_HORIZONTAL
		);
		void set_digits(int digits);
		void set_draw_value(bool draw_value = true);
		void set_value_pos(PositionType pos);
	};

	/// vertical Scale
	class VScale :
		public Scale
	{
	public:
		VScale(Adjustment * adjust) :
			Scale(adjust, ORIENTATION_VERTICAL)
		{}
	};

	/// horizontal Scale
	class HScale :
		public Scale
	{
	public:
		HScale(Adjustment * adjust) :
			Scale(adjust, ORIENTATION_HORIZONTAL)
		{}
	};
}
#endif
