/* scrollbar.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_SCROLLBAR_HH
#define GLTK_SCROLLBAR_HH
#include "entry.hh"
#include "button.hh"
#include "container.hh"
#include "adjustment.hh"
#include "orientation.hh"
namespace gltk {
	/** widget for manipulating an Adjustment
	    \image html ScrollBar.png */
	class Scrollbar :
		virtual public Container
	{
		Adjustment * adjust;
		bool hori; // horizontal?
		Button up;
		Button dn;
		// overrides
		void on_configure_event();
		void on_draw();
		void on_button_event(int button, int state, int x, int y);
		void on_motion_event(int x, int y);

		unsigned spin_wait;
		unsigned spin_interval;
		bool held;
		bool spinning;
		void button_held();
		void pushed_up();
		void pushed_dn();

		// slider
		int slider_z0;
		int slider_sz;
		bool sliding;
		int sliding_start;
		double sliding_vs;
		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;
	public:
		/// construct the Scrollbar on a give Adjustment
		Scrollbar(Adjustment * adjust, Orientation orientation = ORIENTATION_HORIZONTAL);
	};

	class VScrollbar :
		public Scrollbar
	{
	public:
		VScrollbar(Adjustment * adjust) :
			Scrollbar(adjust, ORIENTATION_VERTICAL)
		{}
	};

	class HScrollbar :
		public Scrollbar
	{
	public:
		HScrollbar(Adjustment * adjust) :
			Scrollbar(adjust, ORIENTATION_HORIZONTAL)
		{}
	};
}
#endif
