/* shape.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "shape.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <iostream>
using namespace gltk;

void Shape::on_draw()
{
	int w = get_width();
	int h = get_height();
	switch (shape) {
	case UP_TRIANGLE:
		glColor3f(0, 0, 0);
		glBegin(GL_TRIANGLES);
		glVertex2f(0, h);
		glVertex2f(w / 2.0, 0);
		glVertex2f(w, h);
		glEnd();
		break;
	case DOWN_TRIANGLE:
		glColor3f(0, 0, 0);
		glBegin(GL_TRIANGLES);
		glVertex2f(0, 0);
		glVertex2f(w / 2.0, h);
		glVertex2f(w, 0);
		glEnd();
		break;
	case LEFT_TRIANGLE:
		glColor3f(0, 0, 0);
		glBegin(GL_TRIANGLES);
		glVertex2f(0, h / 2.0);
		glVertex2f(w, h);
		glVertex2f(w, 0);
		glEnd();
		break;
	case RIGHT_TRIANGLE:
		glColor3f(0, 0, 0);
		glBegin(GL_TRIANGLES);
		glVertex2f(0, 0);
		glVertex2f(w, h / 2.0);
		glVertex2f(0, h);
		glEnd();
		break;
	default:
		throw Error("Unknown shape");
	}
}

void Shape::get_preferred_width(int & min, int & nat) const
{
	min = 0;
	nat = width;
}

void Shape::get_preferred_height(int & min, int & nat) const
{
	min = 0;
	nat = height;
}

Shape::Shape(Predefined s, int w, int h) :
	shape(s),
	width(w),
	height(h)
{
}
