/* shape.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_SHAPE_HH
#define GLTK_SHAPE_HH
#include "widget.hh"
namespace gltk {
	/** Widget of simple Shape.
	    These are simple scalable geometric shapes that can be used as
	    icons. */
	class Shape :
		public virtual Widget
	{
	public:
		/// available predefined shapes
		enum Predefined {
			UP_TRIANGLE, ///< upward triangle
			DOWN_TRIANGLE, ///< downward triangle
			LEFT_TRIANGLE, ///< leftward triangle
			RIGHT_TRIANGLE ///< rightward triangle
		};
	private:
		void on_draw();
		void get_preferred_width(int & minimum, int & natural) const;
		void get_preferred_height(int & minimum, int & natural) const;

		Predefined shape;
		int width;
		int height;
	public:
		/// initialize with a shape and sizes
		Shape(
			Predefined shape, ///< select a predefined shape
			int width, ///< default width
			int height ///< default height
		);
	};
}
#endif
