/* spinbutton.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_SPINBUTTON_HH
#define GLTK_SPINBUTTON_HH
#include "entry.hh"
#include "button.hh"
#include "container.hh"
#include "adjustment.hh"
namespace gltk {
	/** a combo of an Entry and two Button%s.
	    \image html SpinButton.png */
	class SpinButton :
		virtual public Container,
		virtual public Adjustment
	{
		Entry tx;
		Button up;
		Button dn;
		// overrides
		void on_configure_event();
		void on_draw();
		void on_value_changed();
		void on_button_event(int button, int state, int x, int y);
		bool on_special_event(int key);

		void value_enter();
		void value_stop();

		unsigned spin_wait;
		unsigned spin_interval;
		double climb_value;
		bool held;
		sigc::connection held_timeout;
		bool spinning;
		bool button_held();
		void pushed_up();
		void pushed_dn();
	protected:
		double climb_rate; ///< acceleration rate when buttons are pressed
		unsigned digits; ///< number of digits after decimal place
		bool snap_to_ticks; ///< snap to multiples of step increment
		bool wrap; ///< wrap to opposite side when limit is reached
	public:
		/// construct with initial #climb_rate and #digits
		SpinButton(
			double climb_rate = 0.0, ///< initial value for #climb_rate
			unsigned digits = 0 ///< initial value for #digits
		);
		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;

		int get_value_as_int() const; ///< get value as an int
		unsigned get_digits() const; ///< get #digits
		void set_digits(unsigned digits); ///< set #digits
		void set_snap_to_ticks(bool snap_to_ticks); ///< set #snap_to_ticks
		void set_increments(double step, double page); ///< set step and page increments
		void set_range(double min, double max); ///< set the range of Adjustment
		void set_wrap(bool wrap = true); ///< wrap value when limits are reached
		void set_width_chars(int n_chars); ///< set the width of Entry field in number of characters
	};
}
#endif
