/* textdraw.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_TEXTDRAW_HH
#define GLTK_TEXTDRAW_HH
#include "widget.hh"
#include "align.hh"
#include <string>
namespace gltk {
	/// a single line text drawing object
	class TextDraw
	{
		std::string text;
		void * font;
	protected:
		/// draw the text at the current raster position
		void draw_text(
			Align xalign = ALIGN_START, ///< horizontal alignment
			Align yalign = ALIGN_START, ///< vertical alignment
			int width_limit = 0 ///< width limit of drawn text
		);
		int text_height() const; ///< get height of text
		int text_width() const; ///< get width of text
		/// get width of a given string
		int text_width(
			std::string const & string ///< string whoes width is to be calculated
		) const;

		virtual void on_set_text() {}
	public:
		TextDraw();
		void set_text(std::string const & text); ///< set the text
		std::string const & get_text() const {return text;} ///< get the text
		void * get_font() const {return font;} ///< get used font
		int get_ascent() const; ///< get the ascent of font used
	};
}
#endif
