/* timeout.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_TIMEOUT_HH
#define GLTK_TIMEOUT_HH
#include "sigc.hh"
namespace gltk {
	/** A timer signal.
	    SignalTimeout is a system signal that can be scheduled to
	    call a slot after a specified amount of time. */
	class SignalTimeout
	{
	public:
		/** Connects a timeout handler.
		    Returning false from the slot will sever the connection. */
		static sigc::connection connect(
			sigc::slot0<bool> const & slot, ///< A slot to call when interval has elapsed.
			unsigned interval ///< The timeout in milliseconds.
		);
		/// Connects a timeout handler that runs only once.
		static void connect_once(sigc::slot0<void> const & slot, unsigned interval);
	};
}
#endif
